/*
 * CanGrow - simply DIY automatic plant grow system (for cannabis).
 * 
 */

/*
 * Includes
 * 
 */

// Libraries
// https://github.com/arduino/ArduinoCore-avr/tree/master/libraries/SPI
#include <SPI.h>
// https://github.com/arduino/ArduinoCore-avr/tree/master/libraries/Wire
#include <Wire.h>
// https://github.com/arduino/ArduinoCore-avr/tree/master/libraries/EEPROM
#include <EEPROM.h>
// https://github.com/esp8266/Arduino/tree/master/libraries/ESP8266WiFi
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
// https://github.com/esp8266/Arduino/tree/master/libraries/ESP8266WebServer
#include <ESP8266WebServer.h>
// https://github.com/adafruit/Adafruit-GFX-Library
#include <Adafruit_GFX.h>
// https://github.com/adafruit/Adafruit_SSD1306
#include <Adafruit_SSD1306.h>
// https://github.com/adafruit/DHT-sensor-library
#include "DHT.h"
// https://github.com/bblanchon/ArduinoJson
#include <ArduinoJson.h>
// https://github.com/arduino-libraries/NTPClient
#include <NTPClient.h>
// https://github.com/PaulStoffregen/Time
#include <TimeLib.h>

// CanGrow header files

#include "CanGrow_PinAssignments.h"
#include "CanGrow_Init.h"
#include "CanGrow_Logo.h"
#include "CanGrow_Sensors.h"

#include "CanGrow_HTML.h"
#include "CanGrow_SysFunctions.h"
#include "CanGrow_WebFunctions.h"



/*
 * Setup
 * 
 */
void setup() {

  // setup pins
  pinMode(PinFAN, OUTPUT);
  pinMode(PINdht, INPUT);
  pinMode(PINwaterlevel, OUTPUT);  
  pinMode(PINsoilmoisture, OUTPUT);  
  pinMode(PinLED, OUTPUT);
  pinMode(PinPUMP, OUTPUT);
  pinMode(PinWIPE, OUTPUT);
  
  
  // set all OUTPUT to low
  digitalWrite(PinFAN, LOW);
  digitalWrite(PINwaterlevel, LOW);
  digitalWrite(PinLED, LOW);
  digitalWrite(PinPUMP, LOW);
  // except PINsoilmoisture
  // PINsoilmoisture is always HIGH and gets LOW in moment of waterlevel measurement
  digitalWrite(PINsoilmoisture, HIGH);
  
  // set PWM frequency lower to avoid annoying noises
  // in combination with 47uF at fan output, 220Hz is kinda sweetspot for a fan
  // but i guess putting just a cap in parallel for voltage regulation isnt a good idea
  // low pass filter or buck converter would be better
  // 220Hz is note A3 btw. 
  analogWriteFreq(220);

  // Start EEPROM
  EEPROM.begin(512);

  // Start Serial
  Serial.begin(115200);

  // Write a line before doing serious output, because before there is some garbage in serial
  // whats get the cursor somewhere over the place
  Serial.println("420");
  Serial.println(".:: CanGrow Start ::.");
    
  // initialise Wire for I2C
  Wire.begin();
  
  // initialise I2C display
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C); // Address 0x3C for 128x64
  display.clearDisplay();
  display.display();

  // set display settings
  display.setTextSize(1);
  display.setTextColor(SSD1306_WHITE, SSD1306_BLACK);
  
  // display Logo
  display.drawBitmap(0, 0, bmpCanGrow_Logo, 128, 32, WHITE);
  display.display();

  // reset chirp
  writeI2CRegister8bit(0x20, 6); //TODO: Do only, when configured
  
  // initialise DHT11
  dht.begin(); //TODO: Do only, when configured
  
  Serial.println("To wipe the EEPROM saved data, set D4 (PinWIPE) to LOW - NOW! (2 seconds left)");
  // wait a few seconds to let the user pull D4 down to wipe EEPROM
  // and we can enjoy the boot screen meanwhile :p
  // meanwhile blink with the led onboad :) 
  // 333 * 6 =~ 2 seconds
  
  display.fillRect(0,36,128,64-36, 0);
  display.setCursor(0,36);
  display.println("To wipe EEPROM pull");
  display.println("D4 (PinWIPE) to GND");
  display.display();
  
  // blink with the onboard LED on D4 (PinWIPE)
  for(byte i = 0; i <= 6 ; i++) {
    if(i % 2) {
      digitalWrite(PinWIPE, LOW);
    } else {
      digitalWrite(PinWIPE, HIGH);
    }
    delay(333);
  }
  // set back to HIGH because thats the default
  digitalWrite(PinWIPE, HIGH);
  //delay(2000);

  // read status from PinWIPE to WIPE
  // when PinWIPE is set to LOW, wipe EEPROM
  if(digitalRead(PinWIPE) == LOW) {
    // wipe EEPROM
    wipeEEPROM();
  }
  
  /*
   * load EEPROM and Setup WiFi
   * 
   * call loadEEPROM() which returns a bool
   * When true, CanGrow is already configured and EEPROM values are applied
   * When false, CanGrow is unconfigured and we need to run the setup assistant
   */
   
   
  // load stored values from EEPROM and check what var configured is returned
  if(loadEEPROM()) {
        
    // connect to wifi
    wifiConnect();
  
  // configured is 0, setup Access Point
  } else {  

    // start an wifi accesspoint
    wifiAp();

  }
  // set web handler
  WebHandler();
  // start webserver
  webserver.begin();
  
  Serial.println(".:: CanGrow Ready ::.");
  delay(1000);
  if(strlen(GrowName) > 0 ) {
    display.clearDisplay();
    display.display();
  }
}


/*
 * 
 * 
 * Loop
 * 
 * 
 */
void loop() {
  // var definition
  unsigned long currentRuntime = millis();

  
  // first we call webserver handle client
  webserver.handleClient();
  
  
  // do every second when everything is configured and grow is started
  if( (configured == true) && (strlen(GrowName) > 0) && (currentRuntime - outputPrevTime >= 1000) ){
    
    // refresh all sensor values
    refreshSensors();
    
    // calculate acutal DayOfGrow
    DayOfGrow = int(ceil(float((timeClient.getEpochTime() - GrowStart) / 60 / 60 / 24)));
    // decide if we are in Veg or Bloom phase of grow
    // when DayOfGrow is larger then DaysVeg we must be in Bloom
    
    // set the actual state of the Grow LED
    // when being in Maintenance Mode and UseRelaisLED not true, 
    // dimm the light
    if(MaintenanceMode == true) {
      if((currentRuntime - MaintenanceStarted <= MaintenanceDuration * 1000 ) && (UseLEDrelais == false)) {
        // in case of being in Maintenance Mode , dimm the grow light when not a relais is used
        setOutput(1, 20);
      } else {
        MaintenanceMode = false;
      }
    } else {
      controlLED();
    }
    displayScreens();
    
    // current time gets previous time for new interval
    outputPrevTime = currentRuntime;
  }
}


  
  
  
  
  
/*
 * 
 * TODO LIST / NOTES
 *
 * 
 * - when PWM for fan is set, set fan speed to regulate humidity and 
 *   temperature, depending on which phase of grow the plant is
 *   (https://www.royalqueenseeds.de/blog-cannabisanbau-im-grow-room-relative-luftfeuchtigkeit-und-temperaturen-n243)
 * - re-organize EEPROM saved values. 
 * - prevent GrowStart to be in the future
 * - maybe let the user configure some screens to display.
 *    
*/
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
/*
 * 
 * 
 * PLAYGROUND / TRASH
 * 
 * 
 */
  
  /*
   
  unsigned long currentTime = millis();
  
  int valSoilmoisture0 = getSoilmoisture(0);
  int valSoilmoisture1 = getSoilmoisture(1);

  float valTemperature0 = getTemperature(0);
  float valTemperature1 = getTemperature(1);

  float valHumidity = getHumidity();
  
  int valWaterlevel = getWaterlevel();

  switch(valWaterlevel) {

    case 0:
      digitalWrite(PinLED, HIGH);  
      digitalWrite(PinPUMP, LOW);
      digitalWrite(PinFAN, LOW);
      break;
    case 1:
      digitalWrite(PinLED, LOW);
      digitalWrite(PinPUMP, HIGH);
      digitalWrite(PinFAN, LOW);
      break;
    case 2:
      digitalWrite(PinLED, LOW);
      digitalWrite(PinPUMP, LOW);
      digitalWrite(PinFAN, HIGH);
      break;
      
  }


// OUTPUT
  if(currentTime - outputPrevTime >= 1000) {
    
  
  // set display cursor to top left
  display.setCursor(0,0);
  // display text
  display.print("I2C: ");
  display.print(valSoilmoisture1);
  display.print(", ");
  display.println(valTemperature1);

  Serial.print("I2C: ");
  Serial.print(valSoilmoisture1);
  Serial.print(", ");
  Serial.println(valTemperature1);

  
  display.print("DHT11: ");
  display.print(valTemperature0);
  display.print(", ");
  display.println(valHumidity);

  Serial.print("DHT11: ");
  Serial.print(valTemperature0);
  Serial.print(", ");
  Serial.println(valHumidity);


  display.print("Water Status: ");
  display.println(valWaterlevel);

  Serial.print("Water Status: ");
  Serial.println(valWaterlevel);

  display.print("ASM: ");
  display.print(valSoilmoisture0);
  display.println(", ");
  
  Serial.print("ASM: ");
  Serial.println(valSoilmoisture0);
  
  // print everything on the display
  display.display();
  
  Serial.println("Test");

  outputPrevTime = currentTime;
*/
  
  
/*  if(D6status == true) {
    digitalWrite(PinLED, LOW);
    digitalWrite(PinPUMP, LOW);
    digitalWrite(PinFAN, LOW);
    D6status = false;
    Serial.println("D6 is off now");
  } else {
    digitalWrite(PinLED, HIGH);
    digitalWrite(PinPUMP, HIGH);
    digitalWrite(PinFAN, HIGH);
    D6status = true;
    Serial.println("D6 is ON now");
  }
  */
  
/*
  for(int dutyCycle = 0; dutyCycle < 255; dutyCycle++){   
    // changing the LED brightness with PWM
    analogWrite(PinLED, dutyCycle);
    delay(1);
  }

  // decrease the LED brightness
  for(int dutyCycle = 255; dutyCycle > 0; dutyCycle--){
    // changing the LED brightness with PWM
    analogWrite(PinLED, dutyCycle);
    delay(1);
  }
*/

  
